# Copyright 2016 - Nokia.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy

from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import forms
from horizon import tables

from mistraldashboard import api
from mistraldashboard.cron_triggers import forms as mistral_forms
from mistraldashboard.cron_triggers import tables as mistral_tables


class OverviewView(generic.TemplateView):
    template_name = 'mistral/cron_triggers/detail.html'
    page_title = _("Cron Trigger Details")
    workflow_url = 'horizon:mistral:workflows:detail'
    list_url = 'horizon:mistral:cron_triggers:index'

    def get_context_data(self, **kwargs):
        context = super(OverviewView, self).get_context_data(**kwargs)
        cron_trigger = {}
        cron_trigger = api.cron_trigger_get(
            self.request,
            kwargs['cron_trigger_name']
        )
        cron_trigger.workflow_url = reverse(
            self.workflow_url,
            args=[cron_trigger.workflow_name]
        )
        cron_trigger.list_url = reverse_lazy(self.list_url)
        breadcrumb = [(cron_trigger.name, reverse(
            'horizon:mistral:cron_triggers:detail',
            args=[cron_trigger.name]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['cron_trigger'] = cron_trigger

        return context


class CreateView(forms.ModalFormView):
    template_name = 'mistral/cron_triggers/create.html'
    modal_header = _("Create Cron Trigger")
    form_id = "create_cron_trigger"
    form_class = mistral_forms.CreateForm
    submit_label = _("Create Cron Trigger")
    submit_url = reverse_lazy("horizon:mistral:cron_triggers:create")
    success_url = reverse_lazy('horizon:mistral:cron_triggers:index')
    page_title = _("Create Cron Trigger")

    def get_form_kwargs(self):
        kwargs = super(CreateView, self).get_form_kwargs()

        return kwargs


class IndexView(tables.DataTableView):
    table_class = mistral_tables.CronTriggersTable
    template_name = 'mistral/cron_triggers/index.html'

    def get_data(self):
        return api.cron_trigger_list(self.request)



# -*- coding: utf-8 -*-
#
# Copyright © 2015 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""File Overrides OpenStack Horizon Cell method to return the whole
 row data using cell_attributes_getter"""

from django import forms
from django import template

from horizon.tables import base

import six


def get_data(self, datum, column, row):
    """Fetches the data to be displayed in this cell."""
    table = row.table
    if column.auto == "multi_select":
        data = ""
        if row.can_be_selected(datum):
            widget = forms.CheckboxInput(check_test=lambda value: False)
            # Convert value to string to avoid accidental type conversion
            data = widget.render('object_ids',
                                 six.text_type(table.get_object_id(datum)),
                                 {'class': 'table-row-multi-select'})
        table._data_cache[column][table.get_object_id(datum)] = data
    elif column.auto == "form_field":
        widget = column.form_field
        if issubclass(widget.__class__, forms.Field):
            widget = widget.widget

        widget_name = "%s__%s" % \
                      (column.name,
                       six.text_type(table.get_object_id(datum)))

        # Create local copy of attributes, so it don't change column
        # class form_field_attributes
        form_field_attributes = {}
        form_field_attributes.update(column.form_field_attributes)
        # Adding id of the input so it pairs with label correctly
        form_field_attributes['id'] = widget_name

        if (template.defaultfilters.urlize in column.filters or
           template.defaultfilters.yesno in column.filters):
                data = widget.render(widget_name,
                                     column.get_raw_data(datum),
                                     form_field_attributes)
        else:
            data = widget.render(widget_name,
                                 column.get_data(datum),
                                 form_field_attributes)
        table._data_cache[column][table.get_object_id(datum)] = data
    elif column.auto == "actions":
        data = table.render_row_actions(datum)
        table._data_cache[column][table.get_object_id(datum)] = data
    else:
        data = column.get_data(datum)
        if column.cell_attributes_getter:
            # Following line is the change: cell_attributes_getter called with
            #  "datum" instead of "data"
            cell_attributes = column.cell_attributes_getter(datum) or {}
            self.attrs.update(cell_attributes)

    return data


def init():
    base.Cell.get_data = get_data



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables
from horizon.utils import filters

from mistraldashboard import api


class CreateWorkbook(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Workbook")
    url = "horizon:mistral:workbooks:select_definition"
    classes = ("ajax-modal",)
    icon = "plus"


class UpdateWorkbook(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Workbook")
    url = "horizon:mistral:workbooks:change_definition"
    classes = ("ajax-modal",)
    icon = "pencil"


class DeleteWorkbook(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Workbook",
            u"Delete Workbooks",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Workbook",
            u"Deleted Workbooks",
            count
        )

    def delete(self, request, workbook_name):
        api.workbook_delete(request, workbook_name)


def tags_to_string(workbook):
    return ', '.join(workbook.tags) if workbook.tags else None


class WorkbooksTable(tables.DataTable):
    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link="horizon:mistral:workbooks:detail"
    )
    tags = tables.Column(tags_to_string, verbose_name=_("Tags"))
    created = tables.Column(
        "created_at",
        verbose_name=_("Created"),
        filters=(
            filters.parse_isotime,
            filters.timesince_or_never
        )
    )
    updated = tables.Column(
        "updated_at",
        verbose_name=_("Updated"),
        filters=(
            filters.parse_isotime,
            filters.timesince_or_never
        )
    )

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "workbooks"
        verbose_name = _("Workbooks")
        table_actions = (
            CreateWorkbook,
            UpdateWorkbook,
            DeleteWorkbook,
            tables.FilterAction
        )
        row_actions = (DeleteWorkbook,)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template.defaultfilters import title
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables
from horizon.utils import filters

from mistraldashboard import api


class CreateWorkflow(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Workflow")
    url = "horizon:mistral:workflows:select_definition"
    classes = ("ajax-modal",)
    icon = "plus"


class UpdateWorkflow(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Workflow")
    url = "horizon:mistral:workflows:change_definition"
    classes = ("ajax-modal",)
    icon = "pencil"


class DeleteWorkflow(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Workflow",
            u"Delete Workflows",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Workflow",
            u"Deleted Workflows",
            count
        )

    def delete(self, request, workflow_name):
        api.workflow_delete(request, workflow_name)


class ExecuteWorkflow(tables.LinkAction):
    name = "execute"
    verbose_name = _("Execute")
    url = "horizon:mistral:workflows:execute"
    classes = ("ajax-modal", "btn-edit")


def tags_to_string(workflow):
    return ', '.join(workflow.tags) if workflow.tags else None


def cut(workflow, length=50):
    inputs = workflow.input

    if inputs and len(inputs) > length:
        return "%s..." % inputs[:length]
    else:
        return inputs


class WorkflowsTable(tables.DataTable):
    name = tables.Column(
        "name",
        verbose_name=_("Name"),
        link="horizon:mistral:workflows:detail"
    )
    id = tables.Column(
        "id",
        verbose_name=_("ID"),
    )
    scope = tables.Column(
        "scope",
        verbose_name=_("Scope"),
        filters=[title],
    )
    definition = tables.Column(
        "",
        verbose_name=_("Definition"),
        empty_value=_("View"),
        link="horizon:mistral:workflows:definition",
        link_classes=("ajax-modal",)
    )

    tags = tables.Column(
        tags_to_string,
        verbose_name=_("Tags")
    )
    inputs = tables.Column(
        cut,
        verbose_name=_("Input"),
        link="horizon:mistral:workflows:input",
        link_classes=("ajax-modal",)
    )
    created = tables.Column(
        "created_at",
        verbose_name=_("Created"),
        filters=(
            filters.parse_isotime,
            filters.timesince_or_never
        )
    )
    updated = tables.Column(
        "updated_at",
        verbose_name=_("Updated"),
        filters=(
            filters.parse_isotime,
            filters.timesince_or_never
        )
    )

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "workflows"
        verbose_name = _("Workflows")
        table_actions = (
            CreateWorkflow,
            UpdateWorkflow,
            DeleteWorkflow,
            tables.FilterAction
        )
        row_actions = (ExecuteWorkflow, DeleteWorkflow)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
import mock

from mistraldashboard.test import helpers as test

INDEX_URL = reverse('horizon:mistral:actions:index')


class ActionsTest(test.TestCase):

    def test_index(self):
        with mock.patch('mistraldashboard.api.pagination_list',
                        return_value=self.mistralclient_actions.list()):
            res = self.client.get(INDEX_URL)

        self.assertTemplateUsed(res, 'mistral/actions/index.html')



# Copyright 2016 - Nokia.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import forms

from mistraldashboard import api
from mistraldashboard.handle_errors import handle_errors


class UpdateForm(forms.SelfHandlingForm):
    action_execution_id = forms.CharField(label=_("Action Execution ID"),
                                          widget=forms.HiddenInput(),
                                          required=False)
    output_source = forms.ChoiceField(
        label=_('Output'),
        help_text=_('Content for output. '
                    'Select either file, raw content or Null value.'),
        choices=[('null', _('<null> (sends empty value)')),
                 ('file', _('File')),
                 ('raw', _('Direct Input'))],
        widget=forms.Select(
            attrs={'class': 'switchable',
                   'data-slug': 'outputsource'}
        ),
        required=False
    )
    output_upload = forms.FileField(
        label=_('Output File'),
        help_text=_('A local output to upload'),
        widget=forms.FileInput(
            attrs={'class': 'switched',
                   'data-switch-on': 'outputsource',
                   'data-outputsource-file': _('Output File')}
        ),
        required=False
    )
    output_data = forms.CharField(
        label=_('Output Data'),
        help_text=_('The raw content for output'),
        widget=forms.widgets.Textarea(
            attrs={'class': 'switched',
                   'data-switch-on': 'outputsource',
                   'data-outputsource-raw': _('Output Data'),
                   'rows': 4}
        ),
        required=False
    )

    state = forms.ChoiceField(
        label=_('State'),
        help_text=_('Select state to update'),
        choices=[('null', _('<null> (sends empty value)')),
                 ('SUCCESS', _('Success')),
                 ('ERROR', _('Error'))],
        widget=forms.Select(
            attrs={'class': 'switchable'}
        ),
        required=False

    )

    def clean(self):
        cleaned_data = super(UpdateForm, self).clean()
        cleaned_data['output'] = None

        if cleaned_data.get('output_upload'):
            files = self.request.FILES
            cleaned_data['output'] = files['output_upload'].read()
        elif cleaned_data.get('output_data'):
            cleaned_data['output'] = cleaned_data['output_data']
        elif cleaned_data.get('output_source') == 'null':
            cleaned_data['output'] = None

        del cleaned_data['output_upload']
        del cleaned_data['output_data']
        del cleaned_data['output_source']

        if cleaned_data['state'] == 'null':
            cleaned_data['state'] = None

        return cleaned_data

    @handle_errors(_("Unable to update Action Execution"), [])
    def handle(self, request, data):
            return api.action_execution_update(
                request,
                data['action_execution_id'],
                data['state'],
                data['output'],
            )



# Copyright 2016 - Nokia.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.urlresolvers import reverse

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from mistraldashboard import api
from mistraldashboard.default.utils import humantime


class CreateCronTrigger(tables.LinkAction):
    name = "create"
    verbose_name = _("Create Cron Trigger")
    url = "horizon:mistral:cron_triggers:create"
    classes = ("ajax-modal",)
    icon = "plus"


class DeleteCronTrigger(tables.DeleteAction):
    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Cron Trigger",
            u"Delete Cron Triggers",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Cron Trigger",
            u"Deleted Cron Triggers",
            count
        )

    def delete(self, request, cron_trigger_name):
        api.cron_trigger_delete(request, cron_trigger_name)


class WorkflowColumn(tables.Column):
    def get_link_url(self, datum):
        workflow_url = "horizon:mistral:workflows:detail"
        obj_id = datum.workflow_name
        return reverse(workflow_url, args=[obj_id])


class CronTriggersTable(tables.DataTable):
    id = tables.Column(
        "id",
        verbose_name=_("ID"),
        link="horizon:mistral:cron_triggers:detail"
    )
    name = tables.Column(
        "name",
        verbose_name=_("Name")
    )
    workflow_name = WorkflowColumn(
        "workflow_name",
        verbose_name=_("Workflow"),
        link=True
    )
    pattern = tables.Column(
        "pattern",
        verbose_name=_("Pattern"),
    )
    next_execution_time = tables.Column(
        "next_execution_time",
        verbose_name=_("Next Execution Time"),
    )
    remaining_executions = tables.Column(
        "remaining_executions",
        verbose_name=_("Remaining Executions"),
    )
    first_execution_time = tables.Column(
        "first_execution_time",
        verbose_name=_("First Execution Time"),
    )
    created_at = tables.Column(
        "created_at",
        verbose_name=_("Created at"),
        filters=[humantime]
    )
    updated_at = tables.Column(
        "updated_at",
        verbose_name=_("Updated at"),
        filters=[humantime]
    )

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "cron trigger"
        verbose_name = _("Cron Trigger")
        table_actions = (
            tables.FilterAction,
            CreateCronTrigger,
            DeleteCronTrigger
        )
        row_actions = (DeleteCronTrigger,)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy

from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions
from horizon import forms
from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import utils

from mistraldashboard import forms as mistral_forms
from mistraldashboard.tasks import tables as mistral_tables


def get_single_task_data(request, **kwargs):
    try:
        task_id = kwargs['task_id']
        task = api.task_get(request, task_id)
    except Exception:
        msg = _('Unable to get task "%s".') % task_id
        redirect = reverse('horizon:mistral:tasks:index')
        exceptions.handle(request, msg, redirect=redirect)

    return task


class ExecutionView(tables.DataTableView):
    table_class = mistral_tables.TaskTable
    template_name = 'mistral/tasks/filtered.html'

    def get_data(self, **kwargs):
        try:
            execution_id = self.kwargs['execution_id']
            tasks = api.task_list(self.request, execution_id)
        except Exception:
            msg = _('Unable to get task by execution id "%s".') % execution_id
            redirect = reverse('horizon:mistral:executions:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return tasks


class OverviewView(generic.TemplateView):
    template_name = 'mistral/tasks/detail.html'
    page_title = _("Task Details")
    workflow_detail_url = 'horizon:mistral:workflows:detail'
    workflow_execution_tasks_url = 'horizon:mistral:executions:tasks'
    execution_url = 'horizon:mistral:executions:detail'
    action_execution_url = 'horizon:mistral:action_executions:task'

    def get_context_data(self, **kwargs):
        context = super(OverviewView, self).get_context_data(**kwargs)
        task = get_single_task_data(self.request, **kwargs)
        task.workflow_detail_url = reverse(self.workflow_detail_url,
                                           args=[task.workflow_name])
        task.execution_url = reverse(self.execution_url,
                                     args=[task.workflow_execution_id])
        task.result = utils.prettyprint(task.result)
        task.published = utils.prettyprint(task.published)
        task.state = utils.label(task.state)
        if task.type == "ACTION":
            task.type_url = reverse(
                self.action_execution_url,
                args=[task.id]
            )
        elif task.type == "WORKFLOW":
            task.type_url = reverse(
                self.workflow_execution_tasks_url,
                args=[task.id]
            )

        breadcrumb = [(task.id, reverse(
            'horizon:mistral:tasks:detail',
            args=[task.id]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['task'] = task

        return context


class CodeView(forms.ModalFormView):
    template_name = 'mistral/default/code.html'
    modal_header = _("Code view")
    form_id = "code_view"
    form_class = mistral_forms.EmptyForm
    cancel_label = "OK"
    cancel_url = reverse_lazy("horizon:mistral:tasks:index")
    page_title = _("Code view")

    def get_context_data(self, **kwargs):
        context = super(CodeView, self).get_context_data(**kwargs)
        column = self.kwargs['column']
        task = get_single_task_data(self.request, **self.kwargs)
        io = {}

        if column == 'result':
            io['name'] = _('Result')
            io['value'] = task.result = utils.prettyprint(task.result)
        elif column == 'published':
            io['name'] = _('Published')
            io['value'] = task.published = utils.prettyprint(task.published)

        context['io'] = io

        return context


class IndexView(tables.DataTableView):
    table_class = mistral_tables.TaskTable
    template_name = 'mistral/tasks/index.html'

    def get_data(self):

        return api.task_list(self.request)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions
from horizon import forms
from horizon import tables

from mistraldashboard.actions import forms as mistral_forms
from mistraldashboard.actions import tables as mistral_tables
from mistraldashboard import api
from mistraldashboard.default import utils


class CreateView(forms.ModalFormView):
    template_name = 'mistral/actions/create.html'
    modal_header = _("Create Action")
    form_id = "create_action"
    form_class = mistral_forms.CreateForm
    submit_label = _("Create")
    submit_url = reverse_lazy("horizon:mistral:actions:create")
    success_url = reverse_lazy('horizon:mistral:actions:index')
    page_title = _("Create Action")


class UpdateView(forms.ModalFormView):
    template_name = 'mistral/actions/update.html'
    modal_header = _("Update Action")
    form_id = "update_action"
    form_class = mistral_forms.UpdateForm
    submit_label = _("Update")
    submit_url = reverse_lazy("horizon:mistral:actions:update")
    success_url = reverse_lazy('horizon:mistral:actions:index')
    page_title = _("Update Action")


class IndexView(tables.DataTableView):
    table_id = "workflow_action"
    table_class = mistral_tables.ActionsTable
    template_name = 'mistral/actions/index.html'

    def has_prev_data(self, table):
        return self._prev

    def has_more_data(self, table):
        return self._more

    def get_data(self):
        actions = []
        prev_marker = self.request.GET.get(
            mistral_tables.ActionsTable._meta.prev_pagination_param,
            None
        )

        if prev_marker is not None:
            sort_dir = 'asc'
            marker = prev_marker
        else:
            sort_dir = 'desc'
            marker = self.request.GET.get(
                mistral_tables.ActionsTable._meta.pagination_param,
                None
            )

        try:
            actions, self._more, self._prev = api.pagination_list(
                entity="actions",
                request=self.request,
                marker=marker,
                sort_keys='name',
                sort_dirs=sort_dir,
                paginate=True,
                reversed_order=True
            )

            if prev_marker is not None:
                actions = sorted(
                    actions,
                    key=lambda action: getattr(
                        action, 'name'
                    ),
                    reverse=False
                )

        except Exception as e:
            self._prev = False
            self._more = False
            msg = _('Unable to retrieve actions list: %s') % str(e)
            exceptions.handle(self.request, msg)

        return actions


class DetailView(generic.TemplateView):
    template_name = 'mistral/actions/detail.html'
    page_title = _("Action Definition")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        action = self.get_data(self.request, **kwargs)
        action.is_system = utils.booleanfield(action.is_system)
        breadcrumb = [(action.name, reverse(
            'horizon:mistral:actions:detail',
            args=[action.id]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['action'] = action

        return context

    def get_data(self, request, **kwargs):
        try:
            action_name = kwargs['action_name']
            action = api.action_get(request, action_name)
        except Exception:
            msg = _('Unable to get action "%s".') % action_name
            redirect = reverse('horizon:mistral:actions:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return action


class RunView(forms.ModalFormView):
    form_class = mistral_forms.RunForm
    template_name = 'mistral/actions/run.html'
    form_id = "run_action"
    success_url = reverse_lazy("horizon:mistral:actions:index")
    submit_label = _("Run")
    modal_header = _("Run Action")
    page_title = _("Run Action")
    submit_url = "horizon:mistral:actions:run"

    def get_initial(self, **kwargs):
        return {'action_name': self.kwargs['action_name']}

    def get_context_data(self, **kwargs):
        context = super(RunView, self).get_context_data(**kwargs)
        context['submit_url'] = reverse(
            self.submit_url,
            args=[self.kwargs["action_name"]]
        )

        return context



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from mistralclient.api.v2 import action_executions
from mistralclient.api.v2 import actions
from mistralclient.api.v2 import cron_triggers
from mistralclient.api.v2 import executions
from mistralclient.api.v2 import tasks
from mistralclient.api.v2 import workbooks
from mistralclient.api.v2 import workflows

from openstack_dashboard.test.test_data import utils as test_data_utils


# Workbooks
WB_DEF = """
---
version: 2.0

name: wb

workflows:
  wf1:
    type: direct
    input:
      - param1
      - param2

    tasks:
      task1:
        action: std.http url="localhost:8989"
        on-success:
          - test_subsequent

      test_subsequent:
        action: std.http url="http://some_url" server_id=1
""".strip()

# Workflows
WF_DEF = """
version: '2.0'

flow:
  tasks:
    task1:
      action: nova.servers_get server="1"
""".strip()


def data(TEST):
    # MistralActions
    TEST.mistralclient_actions = test_data_utils.TestDataContainer()
    action_1 = actions.Action(
        actions.ActionManager(None),
        {'name': 'a',
         'is_system': True,
         'input': 'param1',
         'description': 'my cool action',
         'tags': ['test'],
         'created_at': '1',
         'updated_at': '1'
         }
    )
    TEST.mistralclient_actions.add(action_1)

    # MistralExecutions
    TEST.mistralclient_executions = test_data_utils.TestDataContainer()
    execution_1 = executions.Execution(
        executions.ExecutionManager(None),
        {'id': '123',
         'workflow_name': 'my_wf',
         'description': '',
         'state': 'RUNNING',
         'input': {
             'person': {
                 'first_name': 'John',
                 'last_name': 'Doe'
             }
         }}
    )
    TEST.mistralclient_executions.add(execution_1)

    # Tasks
    TEST.mistralclient_tasks = test_data_utils.TestDataContainer()
    task_1 = tasks.Task(
        tasks.TaskManager(None),
        {'id': '1',
         'workflow_execution_id': '123',
         'name': 'my_task',
         'workflow_name': 'my_wf',
         'state': 'RUNNING',
         'tags': ['deployment', 'demo'],
         'result': {'some': 'result'}})
    TEST.mistralclient_tasks.add(task_1)

    # Workbooks
    TEST.mistralclient_workbooks = test_data_utils.TestDataContainer()
    workbook_1 = workbooks.Workbook(
        workbooks.WorkbookManager(None),
        {'name': 'a',
         'tags': ['a', 'b'],
         'created_at': '1',
         'updated_at': '1',
         'definition': WB_DEF}
    )
    TEST.mistralclient_workbooks.add(workbook_1)

    # Workflows
    TEST.mistralclient_workflows = test_data_utils.TestDataContainer()
    workflow_1 = workflows.Workflow(
        workflows.WorkflowManager(None),
        {'name': 'a',
         'tags': ['a', 'b'],
         'input': 'param',
         'created_at': '1',
         'updated_at': '1',
         'definition': WF_DEF}
    )
    TEST.mistralclient_workflows.add(workflow_1)

    # MistralActionsExecutions
    TEST.mistralclient_action_executions = test_data_utils.TestDataContainer()
    action_executions_1 = action_executions.ActionExecution(
        action_executions.ActionExecutionManager(None),
        {'id': '1',
         'name': 'a',
         'tags': ['a', 'b'],
         'workflow_name': 'my work flow',
         'task_execution_id': '1',
         'task_name': 'b',
         'description': '',
         'created_at': '1',
         'updated_at': '1',
         'accepted': True,
         'state': 'RUNNING'
         }
    )
    TEST.mistralclient_action_executions.add(action_executions_1)

    # MistralCronTriggers
    TEST.mistralclient_cron_triggers = test_data_utils.TestDataContainer()
    cron_triggers_1 = cron_triggers.CronTrigger(
        cron_triggers.CronTriggerManager(None),
        {'id': '1',
         'name': 'a',
         'workflow_name': 'my work flow',
         'pattern': '',
         'next_execution_time': '',
         'remaining_executions': '',
         'first_execution_time': '',
         'created_at': '1',
         'updated_at': '1'
         })
    TEST.mistralclient_cron_triggers.add(cron_triggers_1)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.test_data import utils


def load_test_data(load_onto=None):
    from mistraldashboard.test.test_data import mistral_data
    from openstack_dashboard.test.test_data import exceptions

    # The order of these loaders matters, some depend on others.
    loaders = (
        exceptions.data,
        mistral_data.data,
    )
    if load_onto:
        for data_func in loaders:
            data_func(load_onto)
        return load_onto
    else:
        return utils.TestData(*loaders)



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views import generic

from horizon import exceptions
from horizon import forms
from horizon import tables

from mistraldashboard import api
from mistraldashboard.default import utils
from mistraldashboard import forms as mistral_forms
from mistraldashboard.workflows import forms as workflow_forms
from mistraldashboard.workflows import tables as workflows_tables


def get_single_data(request, workflow_name):
    try:
        workflow = api.workflow_get(request, workflow_name)
    except Exception:
        msg = _('Unable to get workflow "%s".') % workflow_name
        redirect = reverse('horizon:mistral:workflows:index')
        exceptions.handle(request, msg, redirect=redirect)

    return workflow


class IndexView(tables.DataTableView):
    table_class = workflows_tables.WorkflowsTable
    template_name = 'mistral/workflows/index.html'

    def get_data(self):
        return api.workflow_list(self.request)


class DetailView(generic.TemplateView):
    template_name = 'mistral/workflows/detail.html'
    page_title = _("Workflow Definition")

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        workflow = get_single_data(self.request, kwargs['workflow_name'])
        breadcrumb = [(workflow.name, reverse(
            'horizon:mistral:workflows:detail',
            args=[workflow.name]
        ))]

        context["custom_breadcrumb"] = breadcrumb
        context['definition'] = (
            workflow.definition or
            'This workflow was created as part of workbook %s'
            % workflow.name.split('.')[0])

        return context


class CodeView(forms.ModalFormView):
    template_name = 'mistral/default/code.html'
    modal_header = _("Code view")
    form_id = "code_view"
    form_class = mistral_forms.EmptyForm
    cancel_label = "OK"
    cancel_url = reverse_lazy("horizon:mistral:workflows:index")
    page_title = _("Code view")

    def get_context_data(self, **kwargs):
        context = super(CodeView, self).get_context_data(**kwargs)
        workflow = get_single_data(self.request, self.kwargs['workflow_name'])
        io = {}
        column = self.kwargs['column']
        if column == 'definition':
            io['name'] = _('Workflow Definition')
            io['value'] = utils.htmlpre(workflow.definition)
        elif column == 'input':
            io['name'] = _('Workflow Input')
            io['value'] = workflow.input

        context['io'] = io

        return context


class ExecuteView(forms.ModalFormView):
    form_class = workflow_forms.ExecuteForm
    template_name = 'mistral/workflows/execute.html'
    success_url = reverse_lazy("horizon:mistral:executions:index")

    def get_context_data(self, **kwargs):
        context = super(ExecuteView, self).get_context_data(**kwargs)

        context["workflow_name"] = self.kwargs['workflow_name']

        return context

    def get_initial(self, **kwargs):
        return {'workflow_name': self.kwargs['workflow_name']}


class SelectDefinitionView(forms.ModalFormView):
    template_name = 'mistral/workflows/select_definition.html'
    modal_header = _("Create Workflow")
    form_id = "select_definition"
    form_class = workflow_forms.DefinitionForm
    submit_label = _("Validate")
    submit_url = reverse_lazy("horizon:mistral:workflows:select_definition")
    success_url = reverse_lazy('horizon:mistral:workflows:create')
    page_title = _("Select Definition")

    def get_form_kwargs(self):
        kwargs = super(SelectDefinitionView, self).get_form_kwargs()
        kwargs['next_view'] = CreateView

        return kwargs


class ChangeDefinitionView(SelectDefinitionView):
    modal_header = _("Update Workflow")
    submit_url = reverse_lazy("horizon:mistral:workflows:change_definition")
    success_url = reverse_lazy('horizon:mistral:workflows:update')
    page_title = _("Update Definition")

    def get_form_kwargs(self):
        kwargs = super(ChangeDefinitionView, self).get_form_kwargs()
        kwargs['next_view'] = UpdateView

        return kwargs


class CreateView(forms.ModalFormView):
    template_name = 'mistral/workflows/create.html'
    modal_header = _("Create Workflow")
    form_id = "create_workflow"
    form_class = workflow_forms.CreateForm
    submit_label = _("Create")
    submit_url = reverse_lazy("horizon:mistral:workflows:create")
    success_url = reverse_lazy('horizon:mistral:workflows:index')
    page_title = _("Create Workflow")

    def get_initial(self):
        initial = {}

        if 'definition' in self.kwargs:
            initial['definition'] = self.kwargs['definition']

        return initial


class UpdateView(CreateView):
    template_name = 'mistral/workflows/update.html'
    modal_header = _("Update Workflow")
    form_id = "update_workflow"
    form_class = workflow_forms.UpdateForm
    submit_label = _("Update")
    submit_url = reverse_lazy("horizon:mistral:workflows:update")
    page_title = _("Update Workflow")



# Copyright 2014 - StackStorm, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template.loader import render_to_string
import iso8601
import json

TYPES = {
    'SUCCESS': 'label-success',
    'ERROR': 'label-danger',
    'DELAYED': 'label-default',
    'PAUSED': 'label-primary',
    'RUNNING': 'label-info',
}

BOOLEAN_FIELD = {
    'True': {
        'color': 'green',
        'icon': 'fa fa-check'
    },
    'False': {
        'color': 'red',
        'icon': 'fa fa-remove'
    }
}


def label(x):
    return render_to_string("mistral/default/_label.html",
                            {"label": x,
                             "type": TYPES.get(x)})


def booleanfield(x):
    # todo: check undefined instead of the if blocks in view
    # todo: check the red version

    return render_to_string("mistral/default/_booleanfield.html",
                            {"bool": str(x),
                             "type": BOOLEAN_FIELD.get(str(x))})


def humantime(x):
    return render_to_string("mistral/default/_humantime.html",
                            {"datetime": iso8601.parse_date(x)})


def prettyprint(x):
    short = None
    full = json.dumps(json.loads(x), indent=4, ensure_ascii=False)

    lines = full.split('\n')

    if (len(lines) > 5):
        short = '\n'.join(lines[:5] + ['...'])

    return render_to_string("mistral/default/_prettyprint.html",
                            {"full": full, "short": short})


def htmlpre(pre):

    return render_to_string("mistral/default/_preprint.html",
                            {"pre": pre})


def convert_empty_string_to_none(str):
    """Returns None if given string is empty.

    Empty string is default for Django form empty HTML input.
    python-mistral-client does not handle empty strings, only "None" type.

    :param str: string variable
    """

    return str if len(str) != 0 else None
